#!/usr/bin/env python3

message = ("\nNOTE: gubbins_drawer.py is no longer supported. "
           "Instead we recommend using Phandango[1] (http://phandango.net/) to visualise the outputs of Gubbins.\n\n"
           "[1] James Hadfield, Nicholas J Croucher, Richard J Goater, Khalil Abudahab, David M Aanensen, Simon R Harris. "
           "\"Phandango: an interactive viewer for bacterial population genomics\". "
           "Bioinformatics, Volume 34, Issue 2, 15 January 2018, Pages 292–293, "
           "https://doi.org/10.1093/bioinformatics/btx610\n")
print(message)
